#![allow(
    dead_code,
    non_snake_case,
    non_camel_case_types,
    non_upper_case_globals
)]

use libc::{c_char, c_int, c_uint, c_void, uid_t};

#[repr(i32)]
pub(crate) enum Value {
    CAP_CHOWN,
    CAP_DAC_OVERRIDE,
    CAP_DAC_READ_SEARCH,
    CAP_FOWNER,
    CAP_FSETID,
    CAP_KILL,
    CAP_SETGID,
    CAP_SETUID,
    CAP_SETPCAP,
    CAP_LINUX_IMMUTABLE,
    CAP_NET_BIND_SERVICE,
    CAP_NET_BROADCAST,
    CAP_NET_ADMIN,
    CAP_NET_RAW,
    CAP_IPC_LOCK,
    CAP_IPC_OWNER,
    CAP_SYS_MODULE,
    CAP_SYS_RAWIO,
    CAP_SYS_CHROOT,
    CAP_SYS_PTRACE,
    CAP_SYS_PACCT,
    CAP_SYS_ADMIN,
    CAP_SYS_BOOT,
    CAP_SYS_NICE,
    CAP_SYS_RESOURCE,
    CAP_SYS_TIME,
    CAP_SYS_TTY_CONFIG,
    CAP_MKNOD,
    CAP_LEASE,
    CAP_AUDIT_WRITE,
    CAP_AUDIT_CONTROL,
    CAP_SETFCAP,
    CAP_MAC_OVERRIDE,
    CAP_MAC_ADMIN,
    CAP_SYSLOG,
    CAP_WAKE_ALARM,
    CAP_BLOCK_SUSPEND,
    CAP_AUDIT_READ,
    CAP_PERFMON,
    CAP_BPF,
    CAP_CHECKPOINT_RESTORE,
}

impl Into<cap_value_t> for Value {
    fn into(self) -> cap_value_t {
        self as cap_value_t
    }
}

#[repr(u32)]
pub(crate) enum Flag {
    CAP_EFFECTIVE,
    CAP_PERMITTED,
    CAP_INHERITABLE,
}

impl Into<cap_flag_t> for Flag {
    fn into(self) -> cap_flag_t {
        self as cap_flag_t
    }
}

#[repr(u32)]
pub(crate) enum FlagValue {
    CAP_CLEAR,
    CAP_SET,
}

impl Into<cap_flag_t> for FlagValue {
    fn into(self) -> cap_flag_value_t {
        self as cap_flag_value_t
    }
}

#[repr(C)]
#[derive(Copy, Clone)]
pub(crate) struct cap_t {
    pub __user_cap_header_struct: cap_user_header_t,
    pub __user_cap_data_struct: [cap_user_data_t; 2usize],
}

#[repr(C)]
#[derive(Copy, Clone)]

pub(crate) struct cap_user_header_t {
    pub version: u32,
    pub pid: ::libc::c_int,
}

#[repr(C)]
#[derive(Copy, Clone)]
pub(crate) struct cap_user_data_t {
    pub effective: c_uint,
    pub permitted: c_uint,
    pub inheritable: c_uint,
}

pub(crate) type cap_flag_t = c_uint;
pub(crate) type cap_value_t = c_int;
pub(crate) type cap_flag_value_t = c_uint;

// #[link(name = "cap", kind = "dylib")]
#[link(name = "cap", kind = "static")]
extern "C" {
    pub(crate) fn cap_get_proc() -> *mut cap_t;
    pub(crate) fn cap_from_text(arg1: *const c_char) -> *mut cap_t;
    pub(crate) fn cap_set_nsowner(target_caps: *mut cap_t, uid: uid_t) -> c_int;
    pub(crate) fn cap_set_flag(
        arg1: *mut cap_t,
        arg2: cap_flag_t,
        arg3: ::libc::c_int,
        arg4: *const cap_value_t,
        arg5: cap_flag_value_t,
    ) -> ::libc::c_int;
    pub(crate) fn cap_set_proc(arg1: *const cap_t) -> c_int;
    pub(crate) fn cap_set_file(arg1: *const c_char, arg2: *const cap_t) -> c_int;
    pub(crate) fn cap_free(arg1: *mut c_void) -> c_int;
}
